import pandas as pd
import altair as alt


def plot_histogram(data, columns=["all"], num_bins=30):
    """
    Creates histograms for numerical features within a dataframe using Altair.

    Parameters
    ----------
    data : pd.DataFrame
        A pandas dataframe
    columns : list, default=["all"]
        A list of numerical features for which to create histograms, or by default will plot all numerical features in dataframe.
    num_bins : integer, default=30
        Number of bins in histogram plot, default is 30 bins.
    
    Returns
    ------
    plot : altair.Chart object
        An Altair plot for each specified numerical feature
    
    Examples
    --------
    >>> df = pd.DataFrame({'A': [1, 2, 3], 'B' : [1.5, 2.5, 3.5], 'C' : ['one', 'two', 'three']}),
    >>> plot_histogram(df, columns = ['A', 'B'])
    """

    # Check inputs
    if not isinstance(data, pd.DataFrame):
        raise TypeError("'data' should be a pandas.DataFrame object")

    if not isinstance(columns, list):
        raise TypeError("'columns' should be a list object")
        
    if not isinstance(num_bins, int):
        raise TypeError("'num_bins' should be an integer")
        
    if columns == ["all"]:
        numeric_cols = list(data.select_dtypes("number"))
    else:
        numeric_cols = columns

    plot = alt.Chart(data).mark_bar().encode(x=alt.X(alt.repeat(), type="quantitative", bin=alt.Bin(maxbins=num_bins)), y=alt.Y("count()")).properties(height=200, width=200).repeat(repeat=numeric_cols, columns=3)

    return plot


# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['EDAhelper']

package_data = \
{'': ['*']}

install_requires = \
['altair>=4.2.0,<5.0.0']

setup_kwargs = {
    'name': 'edahelper',
    'version': '1.0.3',
    'description': 'Toolset to make EDA easier!',
    'long_description': '# EDAhelper\n[![Documentation Status](https://readthedocs.org/projects/edahelper/badge/?version=latest)](https://edahelper.readthedocs.io/en/latest/?badge=latest)\n[![codecov](https://codecov.io/gh/UBC-MDS/EDAhelper/branch/master/graph/badge.svg?token=2aRO8HaPHn)](https://codecov.io/gh/UBC-MDS/EDAhelper)\n![github workflow](https://github.com/UBC-MDS/EDAhelper/actions/workflows/ci-cd.yml/badge.svg)\n\nTools to make EDA easier!\n\n## About\n\nThis package is aimed at making the EDA process more effective. Basically, we found there were tons of repetitive work when getting a glimpse of the data set. To stop wasting time in repeating procedures, our team came up with the idea to develop a toolkit that includes the following functions:\n\n  1. Clean the data and replace missing values by using the method preferred.\n  2. Provide the description of the data such as the distribution of each column of the data.\n  3. Give the correlation plot between different numeric columns automatically.\n  4. Combine the plots and make them suitable for the report.\n\n## Contributors\n\n- Rowan Sivanandam\n- Steven Leung\n- Vera Cui\n- Jennifer Hoang\n\n## Feature specifications\n\n  1. `preprocess(path, method=None, fill_value=None, read_func=pd.read_csv, **kwarg)` : <br>\n  The function is to preprocess data in txt or csv by dealing with missing values. There are 5 imputation methods provided (None, \'most_frequent\', \'mean\', \'median\', \'constant\'). Finally, it will return the processed data as pandas.DataFrame.\n  2. `column_stats(data, column1, column2 = None, column3 = None, column4 = None)` : <br>\n  The function is to obtain summary statistics of column(s) including count, mean, median, mode, Q1, Q3, variance, standard deviation, correlation. Finally, it will return summary table detailing all statistics and correlations between chosen columns.\n  3. `plot_histogram(data, columns=["all"], num_bins=30)`: : <br>\n  The function is to create histograms for numerical features within a dataframe using Altair. Finally, it will return an Altair plot for each specified continuous feature.\n  4. `numeric_plots(df)` : <br>\n  The function takes a dataframes and plot the possible pairs of the numeric columns using Altair, creating a matrix of correlation plots.\n\n  \n## Related projects\n\nSurely, EDA is not a new topic to data scientists. There are quite a few packages doing similar work on PyPI. However, most of them only include limited functions like just providing descriptive statistics. Our proposal is more of a one-in-all toolkit for EDA. Below is a list of sister-projects.\n\n- [auto-eda](https://pypi.org/project/auto-eda/) : It is an automatic script that generating information in the dataset.\n- [easy-eda](https://pypi.org/project/easy-eda/) : Exploratory Data Analysis.\n- [quick-eda](https://pypi.org/project/quick-eda/) : Important dataframe statistics with a single command.\n- [eda-report](https://pypi.org/project/eda-report/) : A simple program to automate exploratory data analysis and reporting.\n\n## Installation\n\nYou can also use Git to clone the repository from GitHub to install the latest development version:\n```bash\n$ git clone https://github.com/UBC-MDS/EDAhelper.git\n$ cd dist\n$ pip install EDAhelper-3.0.0-py3-none-any.whl\n```\nor install from `PyPI`:\n```bash\n$ pip install edahelper\n```\n\n## Usage\n\nExample usage:\n```python\nfrom EDAhelper import EDAhelper\nEDAhelper.preprocess(\'file_path\')\nEDAhelper.column_stats(df, columns = (\'Date\', PctPopulation\', \'CrimeRatePerPop\'))\nEDAhelper.plot_histogram(df, columns = [\'A\', \'B\'])\nEDAhelper.numeric_plot(df) \n```\n\n## Contributing\n\nInterested in contributing? Check out the contributing guidelines. Please note that this project is released with a Code of Conduct. By contributing to this project, you agree to abide by its terms.\n\n## License\n\n`EDAhelper` was created by Rowan Sivanandam, Steven Leung, Vera Cui, Jennifer Hoang. It is licensed under the terms of the MIT license.\n\n## Credits\n\n`EDAhelper` was created with [`cookiecutter`](https://cookiecutter.readthedocs.io/en/latest/) and the `py-pkgs-cookiecutter` [template](https://github.com/py-pkgs/py-pkgs-cookiecutter).\n',
    'author': 'Rowan Sivanandam, Steven Leung, Vera Cui, Jennifer Hoang',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/UBC-MDS/EDAhelper.git',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



import pandas as pd
import numpy as np


def preprocess(path, method=None, fill_value=None, read_func=pd.read_csv, **kwarg):
    """
    Preprocess data in txt, csv, Excel, etc. by dealing with missing values in numeric columns.

    Parameters
    ----------
    path : str
        The path of the data file.

    method : {None, 'most_frequent', 'mean', 'median', 'constant'},  default=None
        The imputation method.

        If None, then missing values are treated as numpy.NaN.

        If 'mean', then replace missing values using the mean along each column.

        If 'median', then replace missing values using the median along each column.

        If 'most_frequent', then replace missing using the most frequent value along each column. If there is more than one such value, only the smallest is returned.

        If 'constant', then replace missing values with fill_value.

    fill_value : {None, numerical values}, default=None
        When method='constant', fill_value is used to replace all occurrences of missing values.
        If left to the default, fill_value will be 0 when imputing numerical data.

    read_func : panadas.read_* function name, default=pandas.read_csv
        Any function reading data from pandas (e.g. read_csv, read_fwf, read_pickle).

    **kwarg : arbitrary keyword arguments
        Any keyword arguments are defined in @read_func.

    Returns
    -------
    pandas.DataFrame
        The processed table.

    Examples
    -------
    >>> from EDAhelper import EDAhelper
    >>> file_path = 'https://raw.githubusercontent.com/pandas-dev/pandas/main/doc/data/titanic.csv'
    >>> EDAhelper.preprocess(file_path)
    """

    # Input check
    if not isinstance(path, str):
        raise Exception('Err msg: wrong path input')
    # if method not in (None, 'mean', 'median', 'most_frequent', 'constant'):
    #     raise Exception('Err msg: wrong method input')
    if (method == 'constant') & (fill_value is not None) & (not isinstance(fill_value, (float, int))):
        raise Exception("Err msg: wrong fill_value input when method = 'constant'")

    try:
        df = read_func(path, **kwarg)
    except Exception as e:
        raise e

    if method is None:
        return df

    df_num = df.select_dtypes(include='number')
    num_col = df_num.columns
    for col in num_col:
        if df[col].isnull().values.any():
            val_filled = 0
            if method == 'mean':
                val_filled = df[col].mean()
            elif method == 'median':
                val_filled = df[col].median()
            elif method == 'most_frequent':
                val_filled = df[col].mode()[0]
            elif method == 'constant':
                if fill_value:
                    val_filled = fill_value
            else:
                raise Exception('Err msg: wrong method input')

            df.loc[df[col].isnull(), col] = val_filled

    return df
